#import "ViewController.h"
#import "ColorPickViewController.h"
#import "ThumbItem.h"
//#import <SDWebImage/SDWebImage.h>
#import <WebKit/WebKit.h>
#import "IMManager.h"
#import "BoardManager.h"
#import "PopWebView.h"

static NSInteger kSDKAppid = 1400127140;

static NSString *kUpdateElementUrl = nil;
static NSString *kPermissionUrl = nil;
static NSString *kAddWatermarkUrl = nil;
static NSString *kAddVideoFileUrl = nil;
static NSString *kAddH5FileUrl = nil;
static NSString *kAddAudioFileUrl = nil;
static NSString *kAddDyncTextWatermark = nil;
static NSString *kAddStaticTextWatermark = nil;
static NSString *kSetBackgroundImageAngle = nil;
static NSString *kAddTransCodeFileUrl = nil;

// static NSString *kUpdateElementUrl = @"http://localhost:8080/#/UpdateElement";
// static NSString *kPermissionUrl = @"http://localhost:8080/#/Permission";
// static NSString *kAddWatermarkUrl = @"http://localhost:8080/#/AddWatermark";
// static NSString *kAddVideoFileUrl = @"http://localhost:8080/#/AddVideoFile";
// static NSString *kAddH5FileUrl = @"http://localhost:8080/#/AddH5File";
// static NSString *kAddAudioFileUrl = @"http://localhost:8080/#/AddAudioFile";
// static NSString *kAddDyncTextWatermark = @"http://localhost:8080/#/AddDyncTextWatermark";
// static NSString *kAddStaticTextWatermark = @"http://localhost:8080/#/AddStaticTextWatermark";
// static NSString *kSetBackgroundImageAngle = @"http://localhost:8080/#/SetBackgroundImageAngle";

typedef NS_ENUM(NSInteger, TICModule) {
    TICMODULE_IMSDK     = 0,     //IMSDK模块
    TICMODULE_BOARD     = 2,     //BOARD模块
};

@interface ViewController () <TEduBoardDelegate, NSTableViewDelegate, NSTableViewDataSource, NSPopoverDelegate, NSCollectionViewDelegate, NSCollectionViewDataSource, V2TIMSDKListener, IMManagerListener, PopWebViewDelegate, NSTabViewDelegate>
@property (weak) IBOutlet NSPopUpButton *userPopUpButton;
@property (weak) IBOutlet NSPopUpButton *toolTypePopUpButton;
@property (weak) IBOutlet NSPopUpButton *graphStylePopUpButton;
@property (weak) IBOutlet NSPopUpButton *customGraphPopUpButton;
@property (weak) IBOutlet NSPopUpButton *mathToolTypePopUpButton;
@property (weak) IBOutlet NSPopUpButton *magicPenPopUpButton;

@property (weak) IBOutlet NSTableView *boardTableView;
@property (weak) IBOutlet NSTableView *fileTableView;


@property (weak) IBOutlet NSTextField *brushColorTextField;
@property (weak) IBOutlet NSTextField *backgroundColorTextField;
@property (weak) IBOutlet NSTextField *textColorTextField;
@property (weak) IBOutlet NSTextField *highlightColorTextField;

@property (weak) IBOutlet NSTextField *h5BackTextField;

@property (weak) IBOutlet NSTextField *addBoardTextField;

@property (weak) IBOutlet NSView *boardView;
@property (weak) IBOutlet NSTextField *classIdTextField;
@property (weak) IBOutlet NSTextField *imGroupTextField;
@property (weak) IBOutlet NSTextField *nickNameTextField;
@property (weak) IBOutlet NSTextField *messageTextField;
@property (unsafe_unretained) IBOutlet NSTextView *chatTextView;
@property (weak) IBOutlet NSImageView *bgImageView;
@property (weak) IBOutlet NSSlider *scaleSlider;

@property (nonatomic, assign) int setColorIndex;
@property (nonatomic, strong) NSArray *thumbUrls;

@property (weak) IBOutlet NSTextField *imsdkVerLabel;
@property (weak) IBOutlet NSTextField *boardVerLabel;


@property (nonatomic, strong) NSMutableArray *customGraphArray;

//登陆tab按钮
@property (weak) IBOutlet NSView *userGroupContainer;
@property (weak) IBOutlet NSButton *loginButton;
@property (weak) IBOutlet NSButton *logoutButton;


//房间tab按钮
@property (weak) IBOutlet NSView *roomGroupContainer;
@property (weak) IBOutlet NSButton *joinRoomButton;
@property (weak) IBOutlet NSButton *quitRoomButton;

//白板tab按钮
@property (weak) IBOutlet NSButton *undoButton;
@property (weak) IBOutlet NSButton *redoButton;
@property (weak) IBOutlet NSButton *resetStepButton;
@property (weak) IBOutlet NSView *boardGroupContainer;
@property (weak) IBOutlet NSCollectionView *thumbView;
@property (weak) IBOutlet NSTextField *thumbTip;
@property (weak) IBOutlet NSTextField *imageTextFiled;
@property (weak) IBOutlet NSTextField *imgBgTextFiled;

@property (weak) IBOutlet NSTabView *tabView;
@property (strong) NSView *cursorView;

@property (weak) IBOutlet NSTextField *courseUrlField;
@property (weak) IBOutlet NSTextField *customUrlField;
@property (weak) IBOutlet NSButton *courseButton;
@property (weak) IBOutlet NSButton *customButton;
@property (assign) BOOL inJoinRoom;
@property(nonatomic, strong)PopWebView *popWebview;
@property(nonatomic, strong)TEduBoardElement *currentElement;
@property(nonatomic, strong)NSArray<TEduBoardElementInfo *>  *selectElements;
@property(nonatomic, strong)NSArray *userList;

@property(nonatomic, assign)NSTimeInterval lastPermissionToast;
@property(nonatomic, assign)NSTimeInterval lastUsersigToast;
@property(nonatomic, assign)BOOL isShowAlert;
@property(nonatomic, assign)TEduBoardToolType lastToolType;

@property(nonatomic, strong) NSDictionary *permissionDic;
@property(nonatomic, strong) NSString *dataExported;

@property(nonatomic, assign)Boolean autoSelect;
@end

@implementation ViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    kUpdateElementUrl = NSLocalizedString(@"UpdateElementUrl", nil);
    kPermissionUrl = NSLocalizedString(@"PermissionUrl", nil);
    kAddWatermarkUrl = NSLocalizedString(@"AddWatermarkUrl", nil);
    kAddVideoFileUrl = NSLocalizedString(@"AddVideoFileUrl", nil);
    kAddH5FileUrl = NSLocalizedString(@"AddH5File", nil);
    kAddAudioFileUrl = NSLocalizedString(@"AddAudioFile", nil);
    kAddDyncTextWatermark = NSLocalizedString(@"AddDyncTextWatermark", nil);
    kAddStaticTextWatermark = NSLocalizedString(@"AddStaticTextWatermark", nil);
    kSetBackgroundImageAngle = NSLocalizedString(@"SetBackgroundImageAngle", nil);
    kAddTransCodeFileUrl = NSLocalizedString(@"AddTranscodeFile", nil);
    
    [self setupViews];
    [self initTIC];
    [self setupUsers];
    [self setupToolTypes];
    [self setupGraphStyles];
    [self setupMathToolTypes];
    [self setupMagicPenTypes];
    [self setupCustomGraph];
    [self setupThumbView];
    self.tabView.delegate = self;
    //版本信息
    self.imsdkVerLabel.stringValue = [[V2TIMManager sharedInstance] getVersion];
    self.boardVerLabel.stringValue = [TEduBoardController getVersion];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(screenResize) name:NSWindowDidResizeNotification object:nil];
    [IMManager sharedInstance].delegate = self;
}

- (void)dealloc {
    [[IMManager sharedInstance] unInit];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:NSWindowDidResizeNotification object:nil];
    [IMManager sharedInstance].delegate = nil;
}

- (void)tabView:(NSTabView *)tabView didSelectTabViewItem:(nullable NSTabViewItem *)tabViewItem {
    [self enableGroup:self.boardGroupContainer enable:[BoardManager sharedInstance].isEnterRoom ? YES : NO];
}

- (void)updateBtnEnable {
    [self enableGroup:self.userGroupContainer enable:[IMManager sharedInstance].isLogin ? NO : YES];
    [self enableGroup:self.boardGroupContainer enable:[BoardManager sharedInstance].isEnterRoom ? YES : NO];
    [self enableLogin:!([IMManager sharedInstance].isLogin)];
    if(![BoardManager sharedInstance].isEnterRoom && [IMManager sharedInstance].isLogin) {
        [self enableJoin:YES];
    } else {
        [self enableJoin:NO];
    }
    [self enableGroup:self.roomGroupContainer enable:[IMManager sharedInstance].isLogin ];
}

- (void)openPopWebview:(NSString *)url {
    [self closePopWebview];
    self.popWebview = [[PopWebView alloc] init];
    self.popWebview.delegate = self;
    [self.boardView addSubview:self.popWebview];
    self.popWebview.frame = self.boardView.bounds;
    [self.popWebview loadUrl:url];
    
//    // 防止鼠标闪烁
//    TEduBoardToolType toolType = [[BoardManager sharedInstance].boardController getToolType];
//    self.lastToolType = toolType;
//    if (!(toolType == TEDU_BOARD_TOOL_TYPE_POINT_SELECT || toolType == TEDU_BOARD_TOOL_TYPE_RECT_SELECT)) {
//        [[BoardManager sharedInstance].boardController setToolType:TEDU_BOARD_TOOL_TYPE_MOUSE];
//    }
}

- (void)closePopWebview {
    if (self.popWebview) {
        TEduBoardToolType toolType = [[BoardManager sharedInstance].boardController getToolType];
//        if (toolType != self.lastToolType) {
//            [[BoardManager sharedInstance].boardController setToolType:self.lastToolType];
//        }
        [self.popWebview removeFromSuperview];
        [self.popWebview unInit];
        self.popWebview = nil;
    }
}

- (void)reciverH5Msg:(NSString* _Nonnull)action data:(NSDictionary *_Nonnull)data {
    if ([action isEqualToString:@"setElementStyle"]) {
        int styleType = [[data objectForKey:@"styleType"] intValue];
        if (styleType == TEduElementOperatorTypeElementDelete) {
            for (int i = 0; i < self.selectElements.count; i++) {
                [[BoardManager sharedInstance].boardController updateElementById:self.selectElements[i].elementId  type:styleType value:@""];
            }
            [self closePopWebview];
        }
        else if(styleType == TEduElementOperatorTypePosition) {
            NSDictionary *dic = [data objectForKey:@"styleValue"];
            CGPoint point = CGPointMake([[dic objectForKey:@"x"] floatValue], [[dic objectForKey:@"y"] floatValue]);
            [[BoardManager sharedInstance].boardController updateElementById:self.currentElement.elementId position:point];
        }
        else {
            NSString *styleValue = [NSString stringWithFormat:@"%@", [data objectForKey:@"styleValue"]];
            TEduBoardElement *element = [[BoardManager sharedInstance].boardController getElementById:TEduWatermarkTypeText];
            
            TEduBoardElement *element2 = [[BoardManager sharedInstance].boardController getElementById:TEduWatermarkTypeTextFix];
            if((element && [self.currentElement.elementId isEqualToString:element.elementId])
               || (element2 && [self.currentElement.elementId isEqualToString:element2.elementId])) {
                if(styleValue.length > 20) {
                    [self showAlert:TICMODULE_BOARD code:-1 desc:NSLocalizedString(@"WatermarkTextInfo", nil)];
                    return;
                }
            }
            [[BoardManager sharedInstance].boardController updateElementById:self.currentElement.elementId  type:styleType value:styleValue];
        }
    }
    else if ([action isEqualToString:@"addVideoFile"]) {
        NSDictionary *dic = [data objectForKey:@"value"];
        NSString *url = [dic objectForKey:@"url"];
        NSString *title = [dic objectForKey:@"title"];
        [[BoardManager sharedInstance].boardController addVideoFile:url title:title needSwitch:YES];
    }
    else if ([action isEqualToString:@"addAudioFile"]) {
        NSDictionary *dic = [data objectForKey:@"value"];
        NSString *url = [dic objectForKey:@"url"];
        NSString *title = [dic objectForKey:@"title"];
        [[BoardManager sharedInstance].boardController addAudioElement:url withTitle:title];
    }
    else if ([action isEqualToString:@"addH5File"]) {
        NSDictionary *dic = [data objectForKey:@"value"];
        NSString *url = [dic objectForKey:@"url"];
        //        NSString *title = [dic objectForKey:@"title"];
        NSString *angle = [NSString stringWithFormat:@"%@", [dic objectForKey:@"angle"]];
        NSString *boardId = [[BoardManager sharedInstance].boardController addElement:url type:TEDU_BOARD_ELEMENT_H5];
        [[BoardManager sharedInstance].boardController updateElementById:boardId type:TEduElementOperatorTypeRotationAngle value:angle];
    }
    else if ([action isEqualToString:@"setBackgroundImageAngle"]) {
        NSDictionary *dic = [data objectForKey:@"value"];
        NSInteger angle = [[dic objectForKey:@"angle"] intValue];
        [[BoardManager sharedInstance].boardController setBackgroundImageAngle:angle];
    }
    else if ([action isEqualToString:@"addStaticTextWatermark"]) {
        NSDictionary *dic = [data objectForKey:@"value"];
        NSString *url = [dic objectForKey:@"url"];
        NSString *title = [dic objectForKey:@"title"];
        [[BoardManager sharedInstance].boardController addH5File:url title:title needSwitch:YES];
    }
    else if ([action isEqualToString:@"addElement"]) {
        NSDictionary *dic = [data objectForKey:@"value"];
        NSInteger type = [[data objectForKey:@"type"] intValue];
        NSString *content = [dic objectForKey:@"content"];
        if(content.length > 20) {
            [self showAlert:TICMODULE_BOARD code:-1 desc:NSLocalizedString(@"WatermarkTextInfo", nil)];
            return;
        }
        if (type == TEDU_BOARD_ELEMENT_WATERMARK) {
            TEduBoardElementWatermark *params = [[TEduBoardElementWatermark alloc] init];
            params.content = [dic objectForKey:@"content"];
            params.left = [dic objectForKey:@"left"];
            params.top = [dic objectForKey:@"top"];
            params.width = [dic objectForKey:@"width"];
            params.height = [dic objectForKey:@"height"];
            params.opacity = [[dic objectForKey:@"opacity"] floatValue];
            params.deg = [[dic objectForKey:@"deg"] intValue];
            params.fix = [[dic objectForKey:@"fix"] boolValue];
            params.textSize = [dic objectForKey:@"textSize"];
            params.color = [dic objectForKey:@"color"];
            params.interval = [[dic objectForKey:@"interval"] intValue];
            [[BoardManager sharedInstance].boardController addElementWithWatermark:params options:nil];
            [self closePopWebview];
        }
    }
    else if ([action isEqualToString:@"closeWebview"]) {
        [self closePopWebview];
    }
    else if ([action isEqualToString:@"showToast"]) {
        if ([data objectForKey:@"content"]) {
            [self showAlert:TICMODULE_IMSDK code:-1 desc:[data objectForKey:@"content"]];
        }
    }
    else if([action isEqualToString:@"setPermission"]) {
        NSDictionary *content = [data objectForKey:@"content"];
        if(content && [content isKindOfClass:NSDictionary.class]) {
            [self setPermission:content];
        }
        [self closePopWebview];
    }
    else if ([action isEqualToString:@"addTransCodeFile"]) {
        NSDictionary *dic = [data objectForKey:@"value"];
        NSString *url = [dic objectForKey:@"url"];
        NSString *title = [dic objectForKey:@"title"];
        NSInteger pages =  [[dic objectForKey:@"pages"] integerValue];
        
        TEduBoardTranscodeFileResult *result = [[TEduBoardTranscodeFileResult alloc] init];
        result.url = url;
        result.pages = pages;
        result.title = title;
        [[BoardManager sharedInstance].boardController addTranscodeFile:result needSwitch:YES];
    }
}

- (void)setPermission:(NSDictionary *)permissionData {
    self.permissionDic = permissionData;
    TEduBoardController *boardController = [[BoardManager sharedInstance] boardController];
    NSDictionary *permissionMap = @{
        NSLocalizedString(@"SetBgColor", nil): @"Background::Update::Color", NSLocalizedString(@"SetBgFrame", nil): @"Background::Update::Frame",
        NSLocalizedString(@"SetBgImage", nil): @"Background::Update::Image", NSLocalizedString(@"AddWhiteboard", nil): @"Board::Add",
        NSLocalizedString(@"ClearWhiteboard", nil): @"Board::Clear", NSLocalizedString(@"DeleteWhiteboard", nil): @"Board::Delete",
        NSLocalizedString(@"ZoomWhiteboard", nil): @"Board::Scale::*", NSLocalizedString(@"SwitchWhiteboard", nil): @"Board::Switch::Page",
        NSLocalizedString(@"SwitchStep", nil): @"Board::Switch::Step", NSLocalizedString(@"UpdateWbContentFillMode", nil): @"Board::Update::ContentFitMode",
        NSLocalizedString(@"UpdateWbRatio", nil): @"Board::Update::Ratio", NSLocalizedString(@"AddElement", nil): @"Element::Add::*",
        NSLocalizedString(@"DeleteElement", nil): @"Element::Delete::*", NSLocalizedString(@"SelectElement", nil): @"Element::Select::*",
        NSLocalizedString(@"UpdateElement", nil): @"Element::Update::*", NSLocalizedString(@"AddFile", nil): @"File::Add",
        NSLocalizedString(@"ClearFile", nil): @"File::Clear", NSLocalizedString(@"DeleteFile", nil): @"File::Delete",
        NSLocalizedString(@"SwitchFile", nil): @"File::Switch", NSLocalizedString(@"UpdateFile", nil): @"File::Update::*",
        NSLocalizedString(@"SetInstanceAttr", nil): @"Instance::Configure::*", NSLocalizedString(@"ResetWhiteboard", nil): @"Instance::Reset",
        NSLocalizedString(@"Undo", nil): @"Board::Undo::*", NSLocalizedString(@"Redo", nil): @"Board::Redo::*",
    };
    NSMutableArray *havePermissionArray = [NSMutableArray array];
    NSMutableArray *noPermissionArray = [NSMutableArray array];
    NSMutableArray *disableArray = [NSMutableArray array];
    NSArray *array = [permissionData objectForKey:@"backgroundCheckList"];
    if([permissionData objectForKey:@"background"]
       && array && [array isKindOfClass:NSArray.class]) {
        NSArray *background = @[NSLocalizedString(@"SetBgColor", nil), NSLocalizedString(@"SetBgFrame", nil), NSLocalizedString(@"SetBgImage", nil)];
        if([[permissionData objectForKey:@"background"] intValue] == 1) {
            // 开启了背景权限校验
            for (NSString *key in background) {
                NSString *value = [permissionMap objectForKey:key];
                if([array containsObject:key]) {
                    [havePermissionArray addObject:value];
                }
                else {
                    [noPermissionArray addObject:value];
                }
            }
        }
        else {
            // 关闭了背景权限校验
            [disableArray addObject:@"Background::*"];
        }
    }
    
    array = [permissionData objectForKey:@"boardCheckList"];
    if([permissionData objectForKey:@"board"]
       && array && [array isKindOfClass:NSArray.class]) {
        NSArray *board = @[NSLocalizedString(@"AddWhiteboard", nil), NSLocalizedString(@"ClearWhiteboard", nil), NSLocalizedString(@"DeleteWhiteboard", nil), NSLocalizedString(@"ZoomWhiteboard", nil), NSLocalizedString(@"SwitchWhiteboard", nil), NSLocalizedString(@"SwitchStep", nil), NSLocalizedString(@"UpdateWbContentFillMode", nil), NSLocalizedString(@"UpdateWbRatio", nil), NSLocalizedString(@"Undo", nil), NSLocalizedString(@"Redo", nil), NSLocalizedString(@"ResetWhiteboard", nil)];
        if([[permissionData objectForKey:@"board"] intValue] == 1) {
            // 开启了白板权限校验
            for (NSString *key in board) {
                NSString *value = [permissionMap objectForKey:key];
                if([array containsObject:key]) {
                    [havePermissionArray addObject:value];
                }
                else {
                    [noPermissionArray addObject:value];
                }
            }
        }
        else {
            // 关闭了白板权限校验
            [disableArray addObject:@"Board::*"];
        }
    }
    
    array = [permissionData objectForKey:@"elementCheckList"];
    if([permissionData objectForKey:@"element"]
       && array && [array isKindOfClass:NSArray.class]) {
        NSArray *board = @[NSLocalizedString(@"AddElement", nil), NSLocalizedString(@"DeleteElement", nil), NSLocalizedString(@"SelectElement", nil), NSLocalizedString(@"UpdateElement", nil)];
        if([[permissionData objectForKey:@"element"] intValue] == 1) {
            // 开启了元素权限校验
            for (NSString *key in board) {
                NSString *value = [permissionMap objectForKey:key];
                if([array containsObject:key]) {
                    [havePermissionArray addObject:value];
                }
                else {
                    [noPermissionArray addObject:value];
                }
            }
        }
        else {
            // 关闭了元素权限校验
            [disableArray addObject:@"Element::*"];
        }
    }
    
    array = [permissionData objectForKey:@"fileCheckList"];
    if([permissionData objectForKey:@"file"]
       && array && [array isKindOfClass:NSArray.class]) {
        NSArray *board = @[NSLocalizedString(@"AddFile", nil), NSLocalizedString(@"ClearFile", nil), NSLocalizedString(@"DeleteFile", nil), NSLocalizedString(@"SwitchFile", nil), NSLocalizedString(@"UpdateFile", nil)];
        if([[permissionData objectForKey:@"file"] intValue] == 1) {
            // 开启了元素权限校验
            for (NSString *key in board) {
                NSString *value = [permissionMap objectForKey:key];
                if([array containsObject:key]) {
                    [havePermissionArray addObject:value];
                }
                else {
                    [noPermissionArray addObject:value];
                }
            }
        }
        else {
            // 关闭了元素权限校验
            [disableArray addObject:@"File::*"];
        }
    }
    
    array = [permissionData objectForKey:@"instanceCheckList"];
    if([permissionData objectForKey:@"instance"]
       && array && [array isKindOfClass:NSArray.class]) {
        NSArray *board = @[NSLocalizedString(@"SetInstanceAttr", nil)];
        if([[permissionData objectForKey:@"instance"] intValue] == 1) {
            // 开启了实例权限校验
            for (NSString *key in board) {
                NSString *value = [permissionMap objectForKey:key];
                if([array containsObject:key]) {
                    [havePermissionArray addObject:value];
                }
                else {
                    [noPermissionArray addObject:value];
                }
            }
        }
        else {
            // 关闭了实例权限校验
            [disableArray addObject:@"Instance::*"];
        }
    }
    if(havePermissionArray.count > 0) {
        [boardController enablePermissionChecker:havePermissionArray filters:@[ [NSString stringWithFormat:@"operator/%@", @"*"]] ];
    }
    if(noPermissionArray.count > 0) {
        [boardController enablePermissionChecker:noPermissionArray filters:@[@"operator/"]];
    }
    if(disableArray.count > 0) {
        [boardController disablePermissionChecker:disableArray];
    }
}
- (void)screenResize {
    self.boardGroupContainer.frame = CGRectMake(self.boardGroupContainer.frame.origin.x, 20, self.view.frame.size.width - self.boardGroupContainer.frame.origin.x - 20, self.view.frame.size.height - 40);
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.3 *NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        NSView *board = [[BoardManager sharedInstance].boardController getBoardRenderView];
        if (board && board.superview == self.boardView) {
            board.frame = self.boardView.bounds;
        }
    });
    
    
}

#pragma mark - 初始化
- (void)initTIC{
    int sdkAppid = (int)kSDKAppid;
    BOOL sucess = [[IMManager sharedInstance] initSDK:sdkAppid listener:self];
    if (!sucess) {
        NSLog(NSLocalizedString(@"IMInitFailed", nil));
    }
}

#pragma mark - 用户

- (IBAction)onLogin:(id)sender {
    [self addMsgInChatView:NSLocalizedString(@"LoggingIn", nil)];
    NSInteger index = self.userPopUpButton.indexOfSelectedItem;
    NSString *userId = self.userList[index];
    __weak typeof(self) ws = self;
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_HIGH, 0), ^{
        NSString *userSig = [ViewController getUsersig:userId];
        [[IMManager sharedInstance] login:userId userSig:userSig callback:^(int code, NSString * _Nullable desc) {
            [ws showAlert:TICMODULE_IMSDK code:code desc:desc];
            [self addMsgInChatView:NSLocalizedString(@"LoginSuccessJoinRoom", nil)];
            if(code == 0){
                [ws enableGroup:ws.roomGroupContainer enable:YES];
                [ws enableLogin:NO];
                [ws enableJoin:YES];
                TEduBoardResourceConfig *config = [[TEduBoardResourceConfig alloc] init];
                config.sdkAppId = kSDKAppid;
                config.userId = userId;
                [TEduBoardResourceController setConfig:config];
            }
        }];
    });
    
}

+ (NSString *)getUsersig:(NSString *)userId {
    return @"请填写正确的UserSig";
}


- (IBAction)onLogout:(id)sender {
    __weak typeof(self) ws = self;
    [[IMManager sharedInstance] logout:^(int code, NSString * _Nullable desc) {
        [ws showAlert:TICMODULE_IMSDK code:code desc:desc];
        [ws enableGroup:ws.roomGroupContainer enable:NO];
        [ws enableGroup:ws.boardGroupContainer enable:NO];
        [ws enableGroup:ws.userGroupContainer enable:NO];
        [ws enableLogin:YES];
    }];
}

- (IBAction)preloadCourse:(id)sender  {
    NSString *url = [self.courseUrlField stringValue];
    if (url.length <= 0) {
        [self showAlert:TICMODULE_BOARD code:-1 desc:NSLocalizedString(@"CoursewareUrlCannotBeEmpty", nil)];
        return;
    }
    if (![url.lowercaseString hasPrefix:@"http"]) {
        [self showAlert:TICMODULE_BOARD code:-1 desc:NSLocalizedString(@"CoursewareUrlSupportHttpOrHttps", nil)];
        return;
    }
    __weak typeof(self) ws = self;
    ws.courseButton.enabled = false;
    bool result = [TEduBoardResourceController loadResource:url expireTime:259200 callback:^(NSString * _Nonnull url, NSInteger progress, NSError * _Nonnull error) {
        NSLog(@"%@:  下载进度：%ld, err: %@......", url, (long)progress, error);
        [ws.courseButton setTitle:[NSString stringWithFormat:NSLocalizedString(@"CoursewarePercentage", nil), progress]];
        if (progress >= 100 || progress <= -1) {
            ws.courseButton.enabled = ws.joinRoomButton.enabled;
        }
    }];
    if (!result) {
        [self showAlert:TICMODULE_BOARD code:-1 desc:NSLocalizedString(@"CoursewarePreloadFailedCheckUrl", nil)];
    }
}

- (IBAction)preloadCustomResource:(id)sender  {
    NSString *url = [self.customUrlField stringValue];
    if (url.length <= 0) {
        [self showAlert:TICMODULE_BOARD code:-1 desc:NSLocalizedString(@"CustomResourceUrlCannotBeEmpty", nil)];
        return;
    }
    if (![url.lowercaseString hasPrefix:@"http"]) {
        [self showAlert:TICMODULE_BOARD code:-1 desc:NSLocalizedString(@"CustomResourceUrlSupportHttpOrHttps", nil)];
        return;
    }
    __weak typeof(self) ws = self;
    ws.customButton.enabled = false;
    bool result = [TEduBoardResourceController loadCustomResource:url resouceType:TEDU_BOARD_PRELOAD_RESOURCE_TYPE_MEDIA expireTime:259200 callback:^(NSString * _Nonnull url, NSInteger progress, NSError * _Nonnull error) {
        NSLog(@"%@:  下载进度：%ld, err: %@......", url, (long)progress, error);
        [ws.customButton setTitle:[NSString stringWithFormat:NSLocalizedString(@"CustomPercentage", nil), progress]];
        if (progress >= 100 || progress <= -1) {
            ws.customButton.enabled = ws.joinRoomButton.enabled;
        }
    }];
    if (!result) {
        [self showAlert:TICMODULE_BOARD code:-1 desc:NSLocalizedString(@"CustomResourceLoadFailedCheckUrl", nil)];
    }
}
#pragma mark - 房间

- (IBAction)onCreate:(id)sender {
    NSString *classId = self.classIdTextField.stringValue;
    if (classId.length <= 0) {
        return;
    }
    NSString *groupId = self.imGroupTextField.stringValue;
    if (groupId.length <= 0) {
        groupId = classId;
    }
    __weak typeof(self) ws = self;
    [[IMManager sharedInstance] createIMGroup:groupId callback:^(int code, NSString * _Nullable desc) {
        if(code == 0){
        }
        else{
            if(code == 10021){
                desc = NSLocalizedString(@"ClassroomCreatedByOthersJoin", nil);
            }
            else if(code == 10025){
                desc = NSLocalizedString(@"ClassroomCreatedJoin", nil);
            }
        }
        [ws showAlert:TICMODULE_IMSDK code:code desc:desc];
    }];
}
- (IBAction)onDestroy:(id)sender {
    NSString *classId = self.classIdTextField.stringValue;
    if (classId.length <= 0) {
        return;
    }
    NSString *groupId = self.imGroupTextField.stringValue;
    if (groupId.length <= 0) {
        groupId = classId;
    }
    
    __weak typeof(self) ws = self;
    [[IMManager sharedInstance] destroyIMGroup:groupId callback:^(int code, NSString * _Nullable desc) {
        if(code == 0){
        }
        else if(code == 10010){
            desc = NSLocalizedString(@"ClassroomDoesNotExist", nil);
        }
        else if(code == 10007){
            desc = NSLocalizedString(@"NoPermissionToDestroyClassroom", nil);
        }
        [ws showAlert:TICMODULE_IMSDK code:code desc:desc];
    }];
}
- (IBAction)onJoin:(id)sender {
    NSString *classId = self.classIdTextField.stringValue;
    NSString *nickName = self.nickNameTextField.stringValue;
    if (classId.length <= 0) {
        return;
    }
    NSString *groupId = self.imGroupTextField.stringValue;
    if (groupId.length <= 0) {
        groupId = classId;
    }
    if(self.inJoinRoom) {
        [self addMsgInChatView:NSLocalizedString(@"JoiningRoomNoRepeat", nil)];
        return;
    }
    __weak typeof(self) ws = self;
    [self addMsgInChatView:NSLocalizedString(@"JoiningRoom", nil)];
    self.inJoinRoom = YES;
    [[IMManager sharedInstance] joinIMGroup:groupId callback:^(int code, NSString *desc) {
        if (code == 0) {
            [self addMsgInChatView:NSLocalizedString(@"JoinRoomSuccessInitWb", nil)];
            [[BoardManager sharedInstance] creteBoard:[NSString stringWithFormat:@"%ld", kSDKAppid]
                                               userId:[IMManager sharedInstance].userId
                                              userSig:[IMManager sharedInstance].userSig
                                              classId:classId
                                            imGroupId:groupId
                                             nickName:nickName
                                             callback:^(int code, NSString * _Nullable desc, TEduBoardController * _Nullable boardController) {
                if (code == 0) {
                    //加入白板View
                    NSView *board = [boardController getBoardRenderView];
                    board.frame = ws.boardView.bounds;
                    [ws.boardView addSubview:board];
                    //更新按钮状态
                    [ws enableGroup:ws.userGroupContainer enable:NO];
                    [ws enableGroup:ws.boardGroupContainer enable:YES];
                    [ws enableJoin:NO];
                    [ws initToolType];
                }
                [ws showAlert:TICMODULE_BOARD code:code desc:desc];
            }];
            //添加白板事件监听
            [[[BoardManager sharedInstance] boardController] addDelegate:self];
            NSView *webview = [[BoardManager sharedInstance].boardController getBoardRenderView];
            if (NSAppKitVersionNumber > 1500) {
                [webview setValue:@NO forKey:@"drawsBackground"];
            } else {
                [webview setValue:@YES forKey:@"drawsTransparentBackground"];
            }
        }
        else {
            self.inJoinRoom = NO;
            [self addMsgInChatView:[NSString stringWithFormat:NSLocalizedString(@"JoinRoomFailedCodeDesc", nil), code, desc]];
            [ws showAlert:TICMODULE_IMSDK code:code desc:desc];
        }
    }];
}
- (IBAction)onQuit:(id)sender {
    self.inJoinRoom = NO;
    __weak typeof(self) ws = self;
    [self closePopWebview];
    [[BoardManager sharedInstance].boardController removeDelegate:self];
    [[BoardManager sharedInstance] destroyBoard];
    [IMManager sharedInstance].delegate = nil;
    [[IMManager sharedInstance] quitIMGroup:[IMManager sharedInstance].classId callback:^(int code, NSString * _Nullable desc) {
        [ws showAlert:TICMODULE_IMSDK code:code desc:desc];
        [ws resetViews];
        //更新按钮状态
        [ws enableGroup:ws.userGroupContainer enable:YES];
        [ws enableGroup:ws.boardGroupContainer enable:NO];
        [ws enableJoin:YES];
        [ws enableLogin:NO];
    }];
}

#pragma mark - 涂鸦

- (IBAction)onCanDrawChecked:(id)sender {
    NSButton *check = (NSButton *)sender;
    if(check.state == NSControlStateValueOn){
        [[BoardManager sharedInstance].boardController setDrawEnable:YES];
    }
    else{
        [[BoardManager sharedInstance].boardController setDrawEnable:NO];
    }
}

- (IBAction)onPenAutoFitChecked:(id)sender {
    NSButton *check = (NSButton *)sender;
    if(check.state == NSControlStateValueOn){
        [[BoardManager sharedInstance].boardController enablePenAutoFit:YES delay:2000];
    }
    else{
        [[BoardManager sharedInstance].boardController enablePenAutoFit:NO delay:2000];
    }
}

- (IBAction)onNickNameVisiableChecked:(id)sender {
    NSButton *check = (NSButton *)sender;
    if(check.state == NSControlStateValueOn){
        [[BoardManager sharedInstance].boardController setOwnerNickNameVisible:YES];
    }
    else{
        [[BoardManager sharedInstance].boardController setOwnerNickNameVisible:NO];
    }
}

- (IBAction)onSegmentEraserChecked:(id)sender {
    NSButton *check = (NSButton *)sender;
    if(check.state == NSControlStateValueOn){
        [[BoardManager sharedInstance].boardController setPiecewiseErasureEnable:YES];
    }
    else{
        [[BoardManager sharedInstance].boardController setPiecewiseErasureEnable:NO];
    }
}

- (IBAction)onDisablePointEventChecked:(id)sender {
    NSButton *check = (NSButton *)sender;
    if(check.state == NSControlStateValueOn){
        [[BoardManager sharedInstance].boardController disablePointerEventResponding:YES];
    }
    else{
        [[BoardManager sharedInstance].boardController disablePointerEventResponding:NO];
    }
}

- (IBAction)onAutoSelectChecked:(id)sender {
    NSButton *check = (NSButton *)sender;
    self.autoSelect = check.state == NSControlStateValueOn;
}

- (IBAction)onUndo:(id)sender {
    [[BoardManager sharedInstance].boardController undo];
}
- (IBAction)onRedo:(id)sender {
    [[BoardManager sharedInstance].boardController redo];
}
- (IBAction)onClearDraws:(id)sender {
    [[BoardManager sharedInstance].boardController clearDraws];
}
- (IBAction)onClear:(id)sender {
    [[BoardManager sharedInstance].boardController clear];
}
- (IBAction)onClearBackground:(id)sender {
    [[BoardManager sharedInstance].boardController setBackgroundImage:@"" mode:0];
}
- (IBAction)onSetBackgroundImage:(id)sender {
    NSString *url = self.imgBgTextFiled.stringValue;
    if([url length] > 0) {
        [[BoardManager sharedInstance].boardController setBackgroundImage:url mode:TEDU_BOARD_IMAGE_FIT_MODE_CENTER];
    }
    //    NSOpenPanel *panel = [NSOpenPanel openPanel];
    //    [panel setMessage:NSLocalizedString(@"SelectImage", nil)];
    //    [panel beginSheetModalForWindow:self.view.window completionHandler:^(NSModalResponse result) {
    //        if(result == NSModalResponseOK){
    //            NSURL *url = [[panel URLs] firstObject];
    //            NSString *path = [url path];
    //            [[BoardManager sharedInstance].boardController setBackgroundImage:path mode:TEDU_BOARD_IMAGE_FIT_MODE_CENTER];
    //        }
    //    }];
}
- (IBAction)onSetBackgroundH5:(id)sender {
    NSString *url = self.h5BackTextField.stringValue;
    if(url.length > 0){
        [[BoardManager sharedInstance].boardController setBackgroundH5:url];
    }
}


- (IBAction)onSetBrushColor:(id)sender {
    self.setColorIndex = 0;
    [self setColor:self.brushColorTextField];
}
- (IBAction)onSetBackgroundColor:(id)sender {
    self.setColorIndex = 1;
    [self setColor:self.backgroundColorTextField];
}
- (IBAction)onSetTextColor:(id)sender {
    self.setColorIndex = 2;
    [self setColor:self.textColorTextField];
}

- (IBAction)onSetHighlightColor:(id)sender {
    self.setColorIndex = 3;
    [self setColor:self.highlightColorTextField];
}

- (IBAction)onBrushThinChanged:(id)sender {
    int value = ((NSSlider *)sender).intValue;
    [[BoardManager sharedInstance].boardController setBrushThin:value];
}

- (IBAction)onTextSizeChanged:(id)sender {
    int value = ((NSSlider *)sender).intValue;
    [[BoardManager sharedInstance].boardController setTextSize:value];
}

- (IBAction)onTextHeightChanged:(id)sender {
    float value = ((NSSlider *)sender).intValue / 100.0f;
    [[BoardManager sharedInstance].boardController setTextLineHeight:value];
}

- (IBAction)onEraserSizeChanged:(id)sender {
    int value = ((NSSlider *)sender).intValue;
    [[BoardManager sharedInstance].boardController setEraserSize:value];
}

- (IBAction)onBoardScaleChanged:(id)sender {
    int value = ((NSSlider *)sender).intValue;
    [[BoardManager sharedInstance].boardController setBoardScale:value];
}

- (IBAction)onToolTypeChanged:(id)sender {
    NSInteger index = self.toolTypePopUpButton.indexOfSelectedItem;
    if (index >= TEDU_BOARD_TOOL_TYPE_RECT_SELECT) {
        index += 1;
    }
    if (index >= TEDU_BOARD_TOOL_TYPE_ARC) {
        index += 1;
    }
    TEduBoardGraphStyle *style= [[TEduBoardGraphStyle alloc] init];
    style.lineType = TEDU_BOARD_LINE_TYPE_SOLID;
    style.startArrowType = TEDU_BOARD_ARROW_TYPE_NONE;
    style.endArrowType = TEDU_BOARD_ARROW_TYPE_NONE;
    style.fillType = TEduBoardFillTypeNone;
    if (index == TEDU_BOARD_TOOL_TYPE_OVAL_SOLID
        || index == TEDU_BOARD_TOOL_TYPE_RECT_SOLID
        || index == TEDU_BOARD_TOOL_TYPE_SQUARE_SOLID
        || index == TEDU_BOARD_TOOL_TYPE_CIRCLE_SOLID) {
        style.fillType = TEduBoardFillTypeSolid;
    }
    [[BoardManager sharedInstance].boardController setGraphStyle:style];
    [[BoardManager sharedInstance].boardController setToolType:(TEduBoardToolType)index];
    if (index == TEDU_BOARD_TOOL_TYPE_BOARD_CUSTOM_GRAPH) {
        // 自定义图形
        self.customGraphPopUpButton.enabled = YES;
    } else {
        self.customGraphPopUpButton.enabled = NO;
    }

}

- (IBAction)onGraphStyleChanged:(id)sender {
    NSInteger index = self.graphStylePopUpButton.indexOfSelectedItem;
    TEduBoardController *boardController = [BoardManager sharedInstance].boardController;
    if (index == 0) {
        [boardController setGraphStyle:[TEduBoardGraphStyle dashedLineNoArrow]];
        [boardController setToolType: TEDU_BOARD_TOOL_TYPE_LINE];
    }
    else if (index == 1) {
        [boardController setGraphStyle:[TEduBoardGraphStyle solidLineEndArrow]];
        [boardController setToolType: TEDU_BOARD_TOOL_TYPE_LINE];
    }
    else if (index == 2) {
        [boardController setGraphStyle:[TEduBoardGraphStyle dashedLineStartArrow]];
        [boardController setToolType: TEDU_BOARD_TOOL_TYPE_LINE];
    }
    else if (index == 3) {
        [boardController setGraphStyle:[TEduBoardGraphStyle solidLineBothArrow]];
        [boardController setToolType: TEDU_BOARD_TOOL_TYPE_LINE];
    }
    else if (index == 4) {
        [boardController setGraphStyle:[TEduBoardGraphStyle solidTriangle]];
        [boardController setToolType: TEDU_BOARD_TOOL_TYPE_RIGHT_TRIANGLE];
    }
    else if (index == 5) {
        [boardController setGraphStyle:[TEduBoardGraphStyle dashedRect]];
        [boardController setToolType: TEDU_BOARD_TOOL_TYPE_RECT];
    }
    else if (index == 6) {
        //    [array addObject:NSLocalizedString(@"SolidIsoscelesTriangle", nil)];
        TEduBoardGraphStyle *style= [[TEduBoardGraphStyle alloc] init];
        style.fillType = TEduBoardFillTypeSolid;
        [boardController setGraphStyle:style];
        [boardController setToolType: TEDU_BOARD_TOOL_TYPE_ISOSCELES_TRIANGLE];
    }
    else if (index == 7) {
        //    [array addObject:NSLocalizedString(@"SolidRightTriangle", nil)];
        TEduBoardGraphStyle *style= [[TEduBoardGraphStyle alloc] init];
        style.fillType = TEduBoardFillTypeSolid;
        [boardController setGraphStyle:style];
        [boardController setToolType: TEDU_BOARD_TOOL_TYPE_RIGHT_TRIANGLE];
    }
    else if (index == 8) {
        //    [array addObject:NSLocalizedString(@"SolidParallelogram", nil)];
        TEduBoardGraphStyle *style= [[TEduBoardGraphStyle alloc] init];
        style.fillType = TEduBoardFillTypeSolid;
        [boardController setGraphStyle:style];
        [boardController setToolType: TEDU_BOARD_TOOL_TYPE_PARALLELOGRAM];
    }
}

- (IBAction)onMathToolTypeChanged:(id)sender {
    NSInteger index = self.mathToolTypePopUpButton.indexOfSelectedItem;
    [[BoardManager sharedInstance].boardController useMathTool:(TEduMathToolType)(index + 1)];
}

- (IBAction)onMagicPenTypeChanged:(id)sender {
    NSInteger index = self.magicPenPopUpButton.indexOfSelectedItem;
    if(index == 0 ) {
        [[BoardManager sharedInstance].boardController enablePenAutoFit:false delay:0];
        [[BoardManager sharedInstance].boardController setPenAutoFittingMode:TEduBoardPenFittingModeNone];
    }
    else if (index == 1) {
        [[BoardManager sharedInstance].boardController enablePenAutoFit:false delay:0];
        [[BoardManager sharedInstance].boardController setPenAutoFittingMode:TEduBoardPenFittingModeAuto];
    }
    else if (index == 2) {
        [[BoardManager sharedInstance].boardController setPenAutoFittingMode:TEduBoardPenFittingModeNone];
        [[BoardManager sharedInstance].boardController enablePenAutoFit:true delay:2000];
    }
}

- (IBAction)onCustomGraphChanged:(id)sender {
    NSInteger index = self.customGraphPopUpButton.indexOfSelectedItem;
    NSString *url = [self.customGraphArray objectAtIndex:index];
    [[BoardManager sharedInstance].boardController addElement:url type:TEDU_BOARD_ELEMENT_CUSTOM_GRAPH];
}
- (void)setupToolTypes
{
    NSMutableArray *array = [NSMutableArray array];
    [array addObject:NSLocalizedString(@"Mouse", nil)];
    [array addObject:NSLocalizedString(@"Pen", nil)];
    [array addObject:NSLocalizedString(@"Eraser", nil)];
    [array addObject:NSLocalizedString(@"LaserPen", nil)];
    [array addObject:NSLocalizedString(@"StraightLine", nil)];
    [array addObject:NSLocalizedString(@"HollowEllipse", nil)];
    [array addObject:NSLocalizedString(@"HollowRectangle", nil)];
    [array addObject:NSLocalizedString(@"SolidEllipse", nil)];
    [array addObject:NSLocalizedString(@"SolidRectangle", nil)];
    [array addObject:NSLocalizedString(@"PointSelectionTool", nil)];
    //    [array addObject:NSLocalizedString(@"FrameSelectionTool", nil)];
    [array addObject:NSLocalizedString(@"TextTool", nil)];
    [array addObject:NSLocalizedString(@"ZoomMove", nil)];
    [array addObject:NSLocalizedString(@"HollowSquare", nil)];
    [array addObject:NSLocalizedString(@"SolidSquare", nil)];
    [array addObject:NSLocalizedString(@"HollowCircle", nil)];
    [array addObject:NSLocalizedString(@"SolidCircle", nil)];
    [array addObject:NSLocalizedString(@"CustomShape", nil)];

    //    [array addObject:NSLocalizedString(@"CircularArc", nil)];
    [array addObject:NSLocalizedString(@"HighlighterPen", nil)];
    [array addObject:NSLocalizedString(@"RightTriangle", nil)];
    [array addObject:NSLocalizedString(@"IsoscelesTriangle", nil)];
    [array addObject:NSLocalizedString(@"Parallelogram", nil)];
    [array addObject:NSLocalizedString(@"Cube", nil)];
    [array addObject:NSLocalizedString(@"Cylinder", nil)];
    [array addObject:NSLocalizedString(@"Cone", nil)];
    [array addObject:NSLocalizedString(@"CoordinateSystem", nil)];
    [self.toolTypePopUpButton removeAllItems];
    [self.toolTypePopUpButton addItemsWithTitles:array];
}

- (void)setupGraphStyles
{
    NSMutableArray *array = [NSMutableArray array];
    [array addObject:NSLocalizedString(@"DashedLineNoArrow", nil)];
    [array addObject:NSLocalizedString(@"SolidLineSolidArrowEnd", nil)];
    [array addObject:NSLocalizedString(@"DashedLineOrdinaryArrowStart", nil)];
    [array addObject:NSLocalizedString(@"SolidLineSolidArrowStartOrdinaryArrowEnd", nil)];
    [array addObject:NSLocalizedString(@"SolidTriangle", nil)];
    [array addObject:NSLocalizedString(@"DashedHollowRectangle", nil)];
    [array addObject:NSLocalizedString(@"SolidIsoscelesTriangle", nil)];
    [array addObject:NSLocalizedString(@"SolidRightTriangle", nil)];
    [array addObject:NSLocalizedString(@"SolidParallelogram", nil)];
    [self.graphStylePopUpButton removeAllItems];
    [self.graphStylePopUpButton addItemsWithTitles:array];
}

- (void)setupMathToolTypes
{
    NSMutableArray *array = [NSMutableArray array];
    [array addObject:NSLocalizedString(@"StraightRuler", nil)];
    [array addObject:NSLocalizedString(@"RightTriangle", nil)];
    [array addObject:NSLocalizedString(@"IsoscelesRightTriangle", nil)];
    [array addObject:NSLocalizedString(@"Protractor", nil)];
    [array addObject:NSLocalizedString(@"Compass", nil)];
    [self.mathToolTypePopUpButton removeAllItems];
    [self.mathToolTypePopUpButton addItemsWithTitles:array];
}

- (void)setupMagicPenTypes
{
    NSMutableArray *array = [NSMutableArray array];
    [array addObject:NSLocalizedString(@"NoFit", nil)];
    [array addObject:NSLocalizedString(@"FitImmediatelyAfterRelease", nil)];
    [array addObject:NSLocalizedString(@"FitAfterLongPress2s", nil)];
    [self.magicPenPopUpButton removeAllItems];
    [self.magicPenPopUpButton addItemsWithTitles:array];
}

- (void)setupCustomGraph
{
    NSMutableArray *array = [NSMutableArray array];
    [array addObject:NSLocalizedString(@"SelectShape", nil)];
    [array addObject:NSLocalizedString(@"Triangle", nil)];
    [array addObject:NSLocalizedString(@"Heart", nil)];
    [array addObject:NSLocalizedString(@"Pentagram", nil)];
    [self.customGraphPopUpButton removeAllItems];
    [self.customGraphPopUpButton addItemsWithTitles:array];
    self.customGraphArray = [NSMutableArray array];
    [self.customGraphArray addObject:@""];
    [self.customGraphArray addObject:@"https://demo.qcloudtiw.com/resource/triangle.svg"];
    [self.customGraphArray addObject:@"https://demo.qcloudtiw.com/resource/heart.svg"];
    [self.customGraphArray addObject:@"https://demo.qcloudtiw.com/resource/star.svg"];

}

- (void)setColor:(NSTextField *)textField
{
    NSStoryboard *story = [NSStoryboard storyboardWithName:@"Main" bundle:[NSBundle mainBundle]];
    ColorPickViewController *pick = [story instantiateControllerWithIdentifier:@"ColorPickViewController"];
    [pick setDefaultColor:textField.backgroundColor];
    pick.block = ^(NSColor *color){
        textField.backgroundColor = color;
    };
    NSPopover *pop = [[NSPopover alloc] init];
    pop.delegate = self;
    pop.contentViewController = pick;
    pop.behavior = NSPopoverBehaviorTransient;
    [pop showRelativeToRect:textField.bounds ofView:textField preferredEdge:NSRectEdgeMinY];
}

- (void)popoverWillClose:(NSNotification *)notification
{
    NSPopover *pop = notification.object;
    ColorPickViewController *pick = (ColorPickViewController *)pop.contentViewController;
    if(self.setColorIndex == 0){
        [[BoardManager sharedInstance].boardController setBrushColor:[pick getPickColor]];
    }
    else if(self.setColorIndex == 1){
        [[BoardManager sharedInstance].boardController setBackgroundColor:[pick getPickColor]];
    }
    else if(self.setColorIndex == 2){
        [[BoardManager sharedInstance].boardController setTextColor:[pick getPickColor]];
    }
    else if(self.setColorIndex == 3){
        [[BoardManager sharedInstance].boardController setHighlighterColor:[pick getPickColor]];
    }
}

#pragma mark - 白板

- (IBAction)onPreStep:(id)sender {
    [[BoardManager sharedInstance].boardController prevStep];
}
- (IBAction)onNextStep:(id)sender {
    [[BoardManager sharedInstance].boardController nextStep];
}

- (IBAction)onPreBoard:(id)sender {
    BOOL resetStep = (self.resetStepButton.state == NSControlStateValueOn ? YES : NO);
    [[BoardManager sharedInstance].boardController preBoard:resetStep];
}
- (IBAction)onNextBoard:(id)sender {
    BOOL resetStep = (self.resetStepButton.state == NSControlStateValueOn ? YES : NO);
    [[BoardManager sharedInstance].boardController nextBoard:resetStep];
}
- (IBAction)onAddBoard:(id)sender {
    NSString *index = [self.addBoardTextField stringValue];
    // 创建一个 NSNumberFormatter 对象
    NSNumberFormatter *numberFormatter = [[NSNumberFormatter alloc] init];
    // 设置格式为整数
    [numberFormatter setNumberStyle:NSNumberFormatterDecimalStyle];
    // 尝试将字符串转换为 NSNumber 对象
    NSNumber *number = [numberFormatter numberFromString:index];
    // 校验转换结果是否为整数
    if (number == nil) {
        [self showAlert:TICMODULE_BOARD code:-1 desc:@"您输入的新增白板索引无效"];
        return;
    }
    
    if([number intValue] < 0) {
        [self showAlert:TICMODULE_BOARD code:-1 desc:@"您输入的新增白板索引无效"];
        return;
    }
    
    
    [[BoardManager sharedInstance].boardController addBoard:nil model:TEDU_BOARD_IMAGE_FIT_MODE_CENTER type:TEDU_BOARD_BACKGROUND_IMAGE needSwitch:YES index:number.integerValue];
}
- (IBAction)onGotoBoard:(id)sender {
    BOOL resetStep = (self.resetStepButton.state == NSControlStateValueOn ? YES : NO);
    NSInteger index = self.boardTableView.selectedRow;
    if(index >= 0){
        NSArray<NSString *> *boardIds = [[BoardManager sharedInstance].boardController getBoardList];
        [[BoardManager sharedInstance].boardController gotoBoard:boardIds[index] resetStep:resetStep];
    }
}
- (IBAction)onDeleteBoard:(id)sender {
    NSInteger index = self.boardTableView.selectedRow;
    if(index >= 0){
        NSArray<NSString *> *boardIds = [[BoardManager sharedInstance].boardController getBoardList];
        NSString *boardId = boardIds[index];
        if ([[boardId uppercaseString] isEqualToString:@"#DEFAULT"]) {
            NSAlert *alert = [[NSAlert alloc] init];
            [alert setMessageText:NSLocalizedString(@"CannotDeleteDefaultWhiteboard", nil)];
            [alert addButtonWithTitle:NSLocalizedString(@"Confirm", nil)];
            [alert beginSheetModalForWindow:self.view.window completionHandler:nil];
            return;
        }
        [[BoardManager sharedInstance].boardController deleteBoard:boardId];
    }
}
- (IBAction)onAddImageElement:(id)sender {
    NSString *url = self.imageTextFiled.stringValue;
    if(url.length != 0){
        [[BoardManager sharedInstance].boardController addElement:url type:TEDU_BOARD_ELEMENT_IMAGE];
    }
}

- (IBAction)onSetPermisssion:(id)sender {
    [self openPopWebview:kPermissionUrl];
    [self.popWebview sendToH5:@"setPermission" params:self.permissionDic];
}

- (NSArray<NSString *>*)getMediaList:(BOOL)isVideo {
    TEduBoardController *boardController = [BoardManager sharedInstance].boardController;
    NSArray<TEduBoardFileInfo *> *infoList = [boardController getFileInfoList];
    NSMutableArray *result = [NSMutableArray array];
    for (TEduBoardFileInfo *info in infoList) {
        NSArray *list = isVideo ? info.mediaInfo.videoIDList : info.mediaInfo.audioIDList;
        if(list && list.count > 0) {
            [result addObjectsFromArray:list];
        }
    }
    return result;
}

- (IBAction)onPlayPPTVideo:(id)sender {
    TEduBoardController *boardController = [BoardManager sharedInstance].boardController;
    [boardController playH5PPTVideo:[self getMediaList:YES]];
}

- (IBAction)onPausePPTVideo:(id)sender {
    TEduBoardController *boardController = [BoardManager sharedInstance].boardController;
    [boardController pauseH5PPTVideo:[self getMediaList:YES]];
}

- (IBAction)onPlayPPTAudio:(id)sender {
    TEduBoardController *boardController = [BoardManager sharedInstance].boardController;
    [boardController playH5PPTAudio:[self getMediaList:NO]];
}

- (IBAction)onPausePPTAudio:(id)sender {
    TEduBoardController *boardController = [BoardManager sharedInstance].boardController;
    [boardController pauseH5PPTAudio:[self getMediaList:NO]];
}
- (void)onTEBBoardCursorPosition:(CGPoint)position {
    NSLog(@"onTEBBoardCursorPosition, x = %.1f, y= %.1f......", position.x, position.y);
    //    NSView *view = [[BoardManager sharedInstance].boardController getBoardRenderView];
    //    if (self.cursorView == nil) {
    //        self.cursorView = [NSImageView imageViewWithImage:[NSImage imageNamed:@"surfboard"]];
    //        [view addSubview:self.cursorView];
    //    }
    //    [self.cursorView setFrame:CGRectMake(position.x, position.y, 32, 32)];
}

- (void)onTEBBoardScaleChanged:(NSString *)boardId scale:(double)scale xOffset:(double)xOffset yOffset:(double)yOffset {
    NSLog(@"BoardScaleChanged, boardId = %@, scale = %0.2f, xOffset = %0.2f, yOffset = %0.2f", boardId, scale, xOffset, yOffset);
    NSString *msg = [NSString stringWithFormat:NSLocalizedString(@"ReceivedWhiteboardZoomEvent", nil), boardId, scale, xOffset, yOffset];
    [self addMsgInChatView:msg];
}

- (void)onTEBH5PPTMediaStatusChanged:(NSString *)fileId
                             mediaId:(NSString *)mediaId
                              status:(TEduBoardH5PPTMediaStatus)status
                         currentTime:(double)currentTime {
    [self addMsgInChatView:[NSString stringWithFormat:@"H5PPTMediaStatusChanged, fileId = %@, mediaId = %@, status = %@, currentTime = %.1f",
                            fileId,
                            mediaId,
                            status == TEDU_BOARD_H5_PPT_MEDIA_STATUS_PLAY ? @"播放" : @"暂停",
                            currentTime]];
}
#pragma mark - 文件


- (IBAction)onAddFile:(id)sender {
    NSAlert *alert = [[NSAlert alloc] init];
    [alert setMessageText:NSLocalizedString(@"WhiteboardTranscodingDeprecated", nil)];
    [alert addButtonWithTitle:NSLocalizedString(@"Confirm", nil)];
    [alert beginSheetModalForWindow:self.view.window completionHandler:nil];
    return;
    //    NSOpenPanel *panel = [NSOpenPanel openPanel];
    //    [panel setMessage:NSLocalizedString(@"SelectPptPdf", nil)];
    //    [panel beginSheetModalForWindow:self.view.window completionHandler:^(NSModalResponse result) {
    //        if(result == NSModalResponseOK){
    //            NSURL *url = [[panel URLs] firstObject];
    //            NSString *path = [url path];
    //            NSString *ext = [[path pathExtension] lowercaseString];
    //            if([ext isEqualToString:@"ppt"] || [ext isEqualToString:@"pptx"] || [ext isEqualToString:@"pdf"]) {
    //                TEduBoardTranscodeConfig *config = [[TEduBoardTranscodeConfig alloc] init];
    //                config.thumbnailResolution = @"200x200";
    ////                2.6.5废弃
    ////                [[BoardManager sharedInstance].boardController applyFileTranscode:path config:config];
    //            }
    //            else{
    //                [self showAlert:TICMODULE_BOARD code:-1 desc:NSLocalizedString(@"PleaseSelectPptPptxPdfFormatFile", nil)];
    //            }
    //        }
    //    }];
}

- (IBAction)onSwitchFile:(id)sender {
    NSInteger index = self.fileTableView.selectedRow;
    if(index >= 0){
        NSArray<TEduBoardFileInfo *> *files = [[BoardManager sharedInstance].boardController getFileInfoList];
        NSString *fileId = files[index].fileId;
        [[BoardManager sharedInstance].boardController switchFile:fileId];
    }
}
- (IBAction)onDeleteFile:(id)sender {
    NSInteger index = self.fileTableView.selectedRow;
    if(index >= 0){
        NSArray<TEduBoardFileInfo *> *files = [[BoardManager sharedInstance].boardController getFileInfoList];
        NSString *fileId = files[index].fileId;
        if ([[fileId uppercaseString] isEqualToString:@"#DEFAULT"]) {
            NSAlert *alert = [[NSAlert alloc] init];
            [alert setMessageText:NSLocalizedString(@"CannotDeleteDefaultWhiteboard", nil)];
            [alert addButtonWithTitle:NSLocalizedString(@"Confirm", nil)];
            [alert beginSheetModalForWindow:self.view.window completionHandler:nil];
            return;
        }
        [[BoardManager sharedInstance].boardController deleteFile:fileId];
    }
}

- (IBAction)onAddH5File:(id)sender {
    [self openPopWebview:kAddH5FileUrl];
}

- (IBAction)onAddTranscodeFile:(id)sender {
    [self openPopWebview:kAddTransCodeFileUrl];
}

- (IBAction)onAddVideo:(id)sender {
    [self openPopWebview:kAddVideoFileUrl];
}

- (IBAction)onAddWatermark:(id)sender {
    [self openPopWebview:kAddWatermarkUrl];
}

- (IBAction)onAddDyncTextWatermark:(id)sender {
    TEduBoardElement *element = [[BoardManager sharedInstance].boardController getElementById:TEduWatermarkTypeText];
    if (element) {
        // 修改水印
        [self onTEBSelectedElements:@[element]];
    } else {
        // 添加水印
        [self openPopWebview:kAddDyncTextWatermark];
    }
}

- (IBAction)onAddStaticWatermark:(id)sender {
    TEduBoardElement *element = [[BoardManager sharedInstance].boardController getElementById:TEduWatermarkTypeTextFix];
    if (element) {
        // 修改水印
        [self onTEBSelectedElements:@[element]];
    } else {
        // 添加水印
        [self openPopWebview:kAddStaticTextWatermark];
    }
}


- (IBAction)onSetBackgroundImageAngle:(id)sender {
    [self openPopWebview:kSetBackgroundImageAngle];
}

- (IBAction)onExportInLocalMode:(id)sender {
    self.dataExported = [[BoardManager sharedInstance].boardController exportInLocalMode];
    [self showAlert:TICMODULE_BOARD code:0 desc:NSLocalizedString(@"DataSavedLocallyTryImport", nil)];
}

- (IBAction)onImportInLocalMode:(id)sender {
    [[BoardManager sharedInstance].boardController importInLocalMode:self.dataExported];
}

- (IBAction)onAddAudio:(id)sender {
    [self openPopWebview:kAddAudioFileUrl];
}
- (IBAction)onHideElements:(id)sender {
    [[BoardManager sharedInstance].boardController setElementsDisplay:@[@(801)] ids:@[] visible:false];
}

- (IBAction)onShowElements:(id)sender {
    [[BoardManager sharedInstance].boardController setElementsDisplay:@[@(801)] ids:@[] visible:true];
}
- (IBAction)onExportInLocalModeWithFilter:(id)sender {
    self.dataExported = [[BoardManager sharedInstance].boardController exportInLocalModeWithFilter:@"{\"include\":{\"types\":[801]}}"];
    [self showAlert:TICMODULE_BOARD code:0 desc:NSLocalizedString(@"DataSavedLocallyTryImport", nil)];
}
- (IBAction)onImportInLocalModeWithFilter:(id)sender {
    [[BoardManager sharedInstance].boardController importInLocalMode:self.dataExported withFilter:@"{\"include\":{\"types\":[801]}}"];
}

- (IBAction)onSnapshot:(id)sender {
    NSAlert *alert = [[NSAlert alloc] init];
    [alert setMessageText:@"温馨提示"];
    [alert setInformativeText:@"请选择截图模式"];
    [alert addButtonWithTitle:@"所见即所得模式"];
    [alert addButtonWithTitle:@"全尺寸模式（请放大白板后体验）"];
    [alert addButtonWithTitle:@"点错了，关闭"];
    NSModalResponse response = [alert runModal];
    
    NSDate *now = [NSDate date];
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setDateFormat:@"yyyyMMdd_HHmmss"];
    NSString *dateString = [dateFormatter stringFromDate:now];
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *documentsDirectory = [paths firstObject];
        
    NSString *pngPath = [documentsDirectory stringByAppendingPathComponent:[NSString stringWithFormat:@"%@.png", dateString]];
    TEduBoardSnapshotInfo *info = [[TEduBoardSnapshotInfo alloc] init];
    info.path = pngPath;
    switch (response) {
        case NSAlertFirstButtonReturn:
            info.full = false;
            [[BoardManager sharedInstance].boardController snapshot:info];
            break;
        case NSAlertSecondButtonReturn:
            info.full = true;
            [[BoardManager sharedInstance].boardController snapshot:info];
            break;
        case NSAlertThirdButtonReturn:
            break;
        default:
            break;
    }
}
#pragma mark - 白板和文件列表

- (NSInteger)numberOfRowsInTableView:(NSTableView *)tableView
{
    if(self.boardTableView == tableView){
        NSArray *boardIds = [[BoardManager sharedInstance].boardController getBoardList];
        return boardIds.count;
    }
    else if(self.fileTableView == tableView){
        NSArray *files = [[BoardManager sharedInstance].boardController getFileInfoList];
        return files.count;
    }
    return 0;
}

- (nullable NSView *)tableView:(NSTableView *)tableView viewForTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row
{
    if(self.boardTableView == tableView){
        NSArray<NSString *> *boardIds = [[BoardManager sharedInstance].boardController getBoardList];
        NSTableCellView *cell = [tableView makeViewWithIdentifier:@"board" owner:self];
        cell.textField.stringValue = boardIds[row];
        return cell;
    }
    else if(self.fileTableView == tableView){
        NSArray<TEduBoardFileInfo *> *files = [[BoardManager sharedInstance].boardController getFileInfoList];
        NSTableCellView *cell = [tableView makeViewWithIdentifier:@"file" owner:self];
        if(tableColumn == tableView.tableColumns[0]){
            cell.textField.stringValue = files[row].fileId;
        }
        else if(tableColumn == tableView.tableColumns[1]){
            cell.textField.stringValue = files[row].title;
        }
        return cell;
    }
    return nil;
}

- (void)addMsgInChatView:(NSString *)msg {
    self.chatTextView.string = [NSString stringWithFormat:@"%@\n%@", self.chatTextView.string, msg];
    [self.chatTextView scrollRangeToVisible:NSMakeRange(self.chatTextView.string.length, 1)];
}

#pragma mark - 消息

- (IBAction)onSendMessage:(id)sender {
    __weak typeof(self) ws = self;
    NSString *message = self.messageTextField.stringValue;
    if(message.length == 0){
        return;
    }
    [[IMManager sharedInstance] sendGroupTextMessage:message callback:^(int code, NSString * _Nullable desc) {
        if(code == 0){
            // 将自己发送的消息展示在界面上
            NSString *msgInfo = [NSString stringWithFormat:NSLocalizedString(@"Me", nil), message];
            ws.chatTextView.string = [NSString stringWithFormat:@"%@\n%@", ws.chatTextView.string, msgInfo];
            [ws.chatTextView scrollRangeToVisible:NSMakeRange(self.chatTextView.string.length, 1)];
            ws.messageTextField.stringValue = @"";
        }
    }];
}
#pragma mark - 界面
- (void)showAlert:(TICModule)module code:(int)code desc:(NSString *)desc{
    if(self.isShowAlert) {
        return;
    }
    self.isShowAlert = YES;
    NSString *msg = nil;
    if(code == 0){
        msg = NSLocalizedString(@"Success", nil);
    }
    else{
        NSString *moduleStr = nil;
        switch (module) {
            case TICMODULE_IMSDK:
                moduleStr = @"imsdk";
                break;
            case TICMODULE_BOARD:
                moduleStr = @"board";
                break;
            default:
                break;
        }
        msg = [NSString stringWithFormat:NSLocalizedString(@"Failure", nil), moduleStr, code, desc];
    }
    NSAlert *alert = [[NSAlert alloc] init];
    [alert setMessageText:msg];
    [alert addButtonWithTitle:NSLocalizedString(@"Confirm", nil)];
    [alert beginSheetModalForWindow:self.view.window completionHandler:^(NSModalResponse returnCode) {
        self.isShowAlert = NO;
    }];
}

- (void)setupViews
{
    //background
    [self.boardView setWantsLayer:YES];
    self.boardView.layer.backgroundColor = [NSColor whiteColor].CGColor;


    //禁用按钮
    [self enableGroup:self.userGroupContainer enable:NO];
    [self enableGroup:self.roomGroupContainer enable:NO];
    [self enableGroup:self.boardGroupContainer enable:NO];
    //启用登陆
    [self enableLogin:YES];
}

- (void)resetViews
{
    NSView *view = [[BoardManager sharedInstance].boardController getBoardRenderView];
    [view removeFromSuperview];
    [self.fileTableView reloadData];
    [self.boardTableView reloadData];

    //消息
    self.chatTextView.string = @"";
    self.messageTextField.stringValue = @"";
}

- (void)setupUsers
{
    NSMutableArray *users = [NSMutableArray array];
    for(int i = 1; i <= 20; i++) {
        [users addObject:[NSString stringWithFormat:@"tiw_mac_%d", i]];
    }
    self.userList = users;
    [self.userPopUpButton removeAllItems];
    [self.userPopUpButton addItemsWithTitles:users];
}

- (void)enableGroup:(NSView *)group enable:(BOOL)enable
{
    if(group.subviews.count != 0){
        for (NSView *view in group.subviews) {
            [self enableGroup:view enable:enable];
        }
    }
    if([group isKindOfClass:[NSButton class]]){
        ((NSButton *)group).enabled = enable;
    }
    if([group isKindOfClass:[NSSlider class]]){
        ((NSSlider *)group).enabled = enable;
    }
}

- (void)enableLogin:(BOOL)enable
{
    self.loginButton.enabled = enable;
    self.logoutButton.enabled = !enable;
    self.userPopUpButton.enabled = enable;
}

- (void)enableJoin:(BOOL)enable
{
    self.joinRoomButton.enabled = enable;
    self.quitRoomButton.enabled = !enable;
    self.courseButton.enabled = enable;
    self.customButton.enabled = enable;
}

- (void)initToolType {
    [self.toolTypePopUpButton selectItemAtIndex:1];
    self.customGraphPopUpButton.enabled = NO;
}

#pragma mark - 白板事件监听
- (void)onTEBInit {
    self.inJoinRoom = NO;
    [[BoardManager sharedInstance].boardController setOwnerNickNameVisible:YES];
//    [[[BoardManager sharedInstance].boardController getBoardRenderView] setOpaque:NO];
//    [[[BoardManager sharedInstance].boardController getBoardRenderView] setBackgroundColor:[UIColor clearColor]];
    [self addMsgInChatView:NSLocalizedString(@"WhiteboardInitializationSuccess", nil)];
}

- (void)onTEBError:(TEduBoardErrorCode)code msg:(NSString *)msg {
    self.inJoinRoom = NO;
    if (code == TEDU_BOARD_SIGNATURE_EXPIRED) {
        [self closePopWebview];
        [[BoardManager sharedInstance] destroyBoard];
        [[IMManager sharedInstance] kickout];
        [self resetViews];
        //启用登陆
        [self enableGroup:self.userGroupContainer enable:YES];
        [self enableGroup:self.boardGroupContainer enable:NO];
        [self enableJoin:YES];
        [self enableLogin:YES];
        [self showUserSigAlert];
    }
}

- (void)onTEBWarning:(TEduBoardWarningCode)code msg:(NSString *)msg {
    if (code == TEDU_BOARD_WARNING_IMAGE_WATERMARK_ALREADY_EXISTS) {
        [self showAlert:TICMODULE_BOARD code:-1 desc:NSLocalizedString(@"ExistingImageWatermark", nil)];
    } else if(code == TEDU_BOARD_WARNING_VIDEO_ALREADY_EXISTS) {
        [self showAlert:TICMODULE_BOARD code:-1 desc:NSLocalizedString(@"DuplicateFileAddition", nil)];
    } else if(code == TEDU_BOARD_WARNING_H5PPT_ALREADY_EXISTS) {
        [self showAlert:TICMODULE_BOARD code:-1 desc:NSLocalizedString(@"DuplicateFileAddition", nil)];
    } else if(code == TEDU_BOARD_WARNING_H5FILE_ALREADY_EXISTS) {
        [self showAlert:TICMODULE_BOARD code:-1 desc:NSLocalizedString(@"DuplicateFileAddition", nil)];
    } else if(code == TEDU_BOARD_WARNING_IMAGESFILE_ALREADY_EXISTS) {
        [self showAlert:TICMODULE_BOARD code:-1 desc:NSLocalizedString(@"DuplicateFileAddition", nil)];
    } else if(code == TEDU_BOARD_WARNING_IMAGE_COURSEWARE_ALREADY_EXISTS) {
        [self showAlert:TICMODULE_BOARD code:-1 desc:NSLocalizedString(@"DuplicateFileAddition", nil)];
    }
}


- (void)onTEBHistroyDataSyncCompleted
{
    //更新列表
    [self.fileTableView reloadData];
    [self.boardTableView reloadData];
    NSString *fileId = [[BoardManager sharedInstance].boardController getCurrentFile];
    [self loadThumb:fileId];

    self.brushColorTextField.backgroundColor = [[BoardManager sharedInstance].boardController getBrushColor];
    self.textColorTextField.backgroundColor = [[BoardManager sharedInstance].boardController getTextColor];
    self.highlightColorTextField.backgroundColor = [[BoardManager sharedInstance].boardController getHighlighterColor];
    self.backgroundColorTextField.backgroundColor = [[BoardManager sharedInstance].boardController getBackgroundColor];

    WKWebView *view = (WKWebView*)[[BoardManager sharedInstance].boardController getBoardRenderView];
    view.wantsLayer = YES;
//    if (NSAppKitVersionNumber > 1500) {
//        [view setValue:@NO forKey:@"drawsBackground"];
//    } else {
//        [view setValue:@YES forKey:@"drawsTransparentBackground"];
//    }

    for(NSView *subview in [view subviews]) {
        subview.wantsLayer = YES;
        [subview.layer setOpaque:NO];
        [subview.layer setBackgroundColor:[NSColor clearColor].CGColor];
    }
    [view.layer setOpaque:NO];
    [view.layer setBackgroundColor:[NSColor clearColor].CGColor];
    int scale = [[BoardManager sharedInstance].boardController getBoardScale];
    self.scaleSlider.intValue = scale;
    [self addMsgInChatView:NSLocalizedString(@"HistoryDataLoadSuccess", nil)];
}

- (void)onTEBBoardDrawStatusChanged:(TEduBoardDrawStatusChanged)status {
    [self addMsgInChatView: status == TEduBoardDrawStatusStart ? NSLocalizedString(@"DrawingStart", nil) : NSLocalizedString(@"DrawingEnd", nil)];
}

- (void)onTEBRedoStatusChanged:(bool)canRedo
{
    self.redoButton.enabled = canRedo;
}
- (void)onTEBUndoStatusChanged:(bool)canUndo
{
    self.undoButton.enabled = canUndo;
}


- (void)onTEBH5FileStatusChanged:(NSString *)fileId status:(TEduBoardH5FileStatus)status
{
    if(status == TEDU_BOARD_H5_FILE_STATUS_LOADED){
        [self.fileTableView reloadData];
    }
}

- (void)onTEBVideoStatusChanged:(NSString *)fileId status:(TEduBoardVideoStatus)status progress:(CGFloat)progress duration:(CGFloat)duration
{
    if (status == TEDU_BOARD_VIDEO_STATUS_ERROR) {
        [self showAlert:TICMODULE_BOARD code:-1 desc:@"视频播放/加载失败"];
    }
    if(status == TEDU_BOARD_VIDEO_STATUS_LOADED){
        [self.fileTableView reloadData];
    }
}

- (void)onTEBH5PPTStatusChanged:(NSString *)fileId
                         status:(TEduBoardH5PPTStatus)status
                        message:(NSString *)message {
    if (fileId.length > 0 && status == TEDU_BOARD_H5_PPT_STATUS_LOADED) {
        NSDictionary *dic = [[BoardManager sharedInstance].boardController getPPTRemarks:fileId pageIndex:-1];
        [self addMsgInChatView:[dic description]];
    } else {
        [self addMsgInChatView:[NSString stringWithFormat:@"H5PPTStatusChanged, fileId = %@, status = %ld", fileId, (long)status]];
    }
}

- (void)onTEBFileTranscodeProgress:(TEduBoardTranscodeFileResult *)result path:(NSString *)path errorCode:(NSString *)errorCode errorMsg:(NSString *)errorMsg
{
    NSString *fileName = [path lastPathComponent];
    NSString *statusStr = nil;
    if(result.status == TEDU_BOARD_FILE_TRANSCODE_UPLOADING){
        statusStr = [NSString stringWithFormat:NSLocalizedString(@"Uploading", nil), result.progress];
    }
    else if(result.status == TEDU_BOARD_FILE_TRANSCODE_CREATED){
        statusStr = NSLocalizedString(@"CreateTranscodingTask", nil);
    }
    else if(result.status == TEDU_BOARD_FILE_TRANSCODE_QUEUED){
        statusStr = NSLocalizedString(@"Queuing", nil);
    }
    else if(result.status == TEDU_BOARD_FILE_TRANSCODE_PROCESSING){
        statusStr = [NSString stringWithFormat:NSLocalizedString(@"Transcoding", nil), result.progress];
    }
    else if(result.status == TEDU_BOARD_FILE_TRANSCODE_FINISHED){
        statusStr = NSLocalizedString(@"TranscodingComplete", nil);
        [[BoardManager sharedInstance].boardController addTranscodeFile:result needSwitch:true];

    }
    else if(result.status == TEDU_BOARD_FILE_TRANSCODE_ERROR){
        statusStr = [NSString stringWithFormat:NSLocalizedString(@"TranscodingError", nil), errorCode, errorMsg];
    }

    NSString *msg = [NSString stringWithFormat:@"%@ %@", fileName, statusStr];
    self.chatTextView.string = [NSString stringWithFormat:@"%@\n%@", self.chatTextView.string, msg];
    [self.chatTextView scrollRangeToVisible:NSMakeRange(self.chatTextView.string.length, 1)];
}

- (void)onTEBAddTranscodeFile:(NSString *)fileId
{
    [self.fileTableView reloadData];
}

- (void)onTEBAddBoard:(NSArray *)boardIds fileId:(NSString *)fileId
{
    [self.boardTableView reloadData];
}
- (void)onTEBDeleteFile:(NSArray *)fileId
{
    [self.fileTableView reloadData];
}
- (void)onTEBDeleteBoard:(NSString *)boardId fileId:(NSString *)fileId
{
    [self.boardTableView reloadData];
}
- (void)onTEBBackgroundH5StatusChanged:(NSString *)boardId url:(NSString *)url status:(TEduBoardBackgroundH5Status)status
{
    NSLog(@"onTEBBackgroundH5StatusChanged boardId=%@, url=%@, status=%ld", boardId, url, status);
}

- (void)onTEBFileUploadStatus:(NSString *)path status:(TEduBoardUploadStatus)status statusMsg:(NSString *)statusMsg
{
    NSLog(@"onTEBFileUploadStatus path=%@, status=%ld, statusMsg=%@", path, status, statusMsg);
}

- (void)onTEBImageStatusChanged:(NSString *)boardId url:(NSString *)url status:(TEduBoardImageStatus)status
{
    NSLog(@"onTEBImageStatusChanged boardId=%@, url=%@, status=%ld", boardId, url, status);
}

- (void)onTEBFileUploadProgress:(NSString *)path currentBytes:(int)currentBytes totalBytes:(int)totalBytes uploadSpeed:(int)uploadSpeed
{
    NSLog(@"onTEBFileUploadProgress path=%@, currentBytes=%d, totalBytes=%d, uploadSpeed=%d", path, currentBytes, totalBytes, uploadSpeed);
}


- (void)onTEBSelectedElements:(NSArray<TEduBoardElementInfo *> *) selectedElements {
    self.selectElements = selectedElements;
    if (selectedElements.count > 0) {
        TEduBoardElementInfo *info = selectedElements[0];
        self.currentElement = [[BoardManager sharedInstance].boardController getElementById:info.elementId];
        
        NSMutableDictionary *params = [NSMutableDictionary dictionary];

        [params setObject:@(self.currentElement.type) forKey:@"type"];
        if (self.currentElement.title) {
            [params setObject:self.currentElement.title forKey:@"title"];
        }

        if (self.currentElement.style) {
            if (self.currentElement.style.borderColor) {
                NSString *borderColorKey = [NSString stringWithFormat:@"%ld", (long)TEduElementOperatorTypeBorderColor];
                [params setObject:self.currentElement.style.borderColor forKey:borderColorKey];
            }
            NSString *borderWidthKey = [NSString stringWithFormat:@"%ld", (long)TEduElementOperatorTypeBorderWidth];
            [params setObject:@(self.currentElement.style.borderWidth) forKey:borderWidthKey];

            NSString *borderTypeKey = [NSString stringWithFormat:@"%ld", (long)TEduElementOperatorTypeBorderLineType];
            [params setObject:@(self.currentElement.style.borderType) forKey:borderTypeKey];
        }
        NSString *positionKey = [NSString stringWithFormat:@"%ld", (long)TEduElementOperatorTypePosition];
        [params setObject:@{@"x": @(self.currentElement.boundingbox.origin.x), @"y": @(self.currentElement.boundingbox.origin.y)} forKey:positionKey];

        NSString *textSizeKey = [NSString stringWithFormat:@"%ld", (long)TEduElementOperatorTypeTextSize];
        [params setObject:@(self.currentElement.textSize) forKey:textSizeKey];
        
        NSString *contentKey = [NSString stringWithFormat:@"%ld", (long)TEduElementOperatorTypeContent];
        [params setObject:self.currentElement.content forKey:contentKey];
        
        NSString *opacityKey = [NSString stringWithFormat:@"%ld", (long)TEduElementOperatorTypeOpacity];
        [params setObject:@(self.currentElement.opacity) forKey:opacityKey];
        [self openPopWebview:kUpdateElementUrl];
        [self.popWebview sendToH5:@"setElementInfo" params:params];
        
    }
}

- (void)onTEBElementsPositonChanged:(NSArray<TEduBoardElementInfo *> *)elements {
    NSLog(@"%@", elements);
}

- (void)onTEBBoardPermissionDenied:(NSString *)permission {
    NSTimeInterval time = [[NSDate date] timeIntervalSince1970];
    if(time - self.lastPermissionToast > 5.0) {
        NSString *text = [NSString stringWithFormat:NSLocalizedString(@"LackOfOperationPermission", nil), permission];
//        [[JMToast sharedToast] showDialogWithMsg:text];
        [self showAlert:TICMODULE_BOARD code:-1 desc:text];
        self.lastPermissionToast = time;
    }
}

- (void)onTEBAudioStatusChanged:(NSString *)elementId
                         status:(TEduBoardAudioStatus)status
                        progress:(CGFloat)progress
                       duration:(CGFloat)duration {
    if (TEDU_BOARD_AUDIO_STATUS_ERROR == status) {
        [self showAlert:TICMODULE_BOARD code:-1 desc:NSLocalizedString(@"AudioPlaybackError", nil)];
    }
}

- (void)onTEBAddElement:(NSString *)elementId url:(NSString *)url type:(TEduBoardElementType)type {
    NSString *msg = [NSString stringWithFormat:@"白板元素添加成功， elementId = %@， type = %d", elementId, (int)type];
    [self addMsgInChatView:msg];
    
    if (self.autoSelect && type == 13) {
        [[BoardManager sharedInstance].boardController autoSelectedElement:elementId];
    }
}

- (void)onTEBBoardH5pptDownGrade:(NSString *)filedId slideIndex:(NSInteger)slideIndex {
    [[BoardManager sharedInstance].boardController setDownGrade:filedId slideIndex:slideIndex enable:YES];
    [self addMsgInChatView:[NSString stringWithFormat:@"PPT加载超时(fid=%@)，将进行降级渲染", filedId]];
}

- (void)onTEBSnapshot:(NSString *)path errorCode:(TEduBoardErrorCode)code errorMsg:(NSString *)msg {
    if(code == 0) {
        [self addMsgInChatView:[NSString stringWithFormat:@"截图成功，图片保存在%@", path]];
    } else {
        [self addMsgInChatView:[NSString stringWithFormat:@"截图失败, code = %ld, msg = %@", (long)code, msg]];
    }

}
#pragma mark - 用户状态监听
- (void)onKickedOffline
{
    [self showAlert:TICMODULE_IMSDK code:-1 desc:NSLocalizedString(@"AccountKicked", nil)];
    [self closePopWebview];
    [[BoardManager sharedInstance] destroyBoard];
    [[IMManager sharedInstance] kickout];
    [self resetViews];
    //启用登陆
    [self enableGroup:self.userGroupContainer enable:YES];
    [self enableGroup:self.boardGroupContainer enable:NO];
    [self enableJoin:YES];
    [self enableLogin:YES];
}

- (void)onUserSigExpired
{
    [self closePopWebview];
    [[BoardManager sharedInstance] destroyBoard];
    [[IMManager sharedInstance] logout:nil];
    [self resetViews];
    //启用登陆
    [self enableGroup:self.userGroupContainer enable:YES];
    [self enableGroup:self.boardGroupContainer enable:NO];
    [self enableJoin:YES];
    [self enableLogin:YES];
    [self showUserSigAlert];
}

- (void)showUserSigAlert {
    NSTimeInterval time = [[NSDate date] timeIntervalSince1970];
    if(time - self.lastUsersigToast > 5.0) {
        [self showAlert:TICMODULE_IMSDK code:-1 desc:NSLocalizedString(@"UserSigExpired", nil)];
        self.lastUsersigToast = time;
    }
}

#pragma mark - 消息监听
- (void)onRecvGroupTextMessage:(NSString *)text groupId:(NSString *)groupId fromUserId:(NSString *)fromUserId
{
    NSString *msgInfo = [NSString stringWithFormat:@"[%@] %@",fromUserId, text];
    self.chatTextView.string = [NSString stringWithFormat:@"%@\n%@",self.chatTextView.string, msgInfo];
    [self.chatTextView scrollRangeToVisible:NSMakeRange(self.chatTextView.string.length, 1)];
}

- (void)onRecvCTCTextMessage:(NSString *)text fromUserId:(NSString *)fromUserId;
{
    NSString *msgInfo = [NSString stringWithFormat:@"[%@](私聊) %@",fromUserId, text];
    self.chatTextView.string = [NSString stringWithFormat:@"%@\n%@",self.chatTextView.string, msgInfo];
    [self.chatTextView scrollRangeToVisible:NSMakeRange(self.chatTextView.string.length, 1)];
}


#pragma mark -缩略图

- (void)onTEBSwitchFile:(NSString *)fileId
{
    [self loadThumb:fileId];
}

- (void)setupThumbView{
    self.thumbTip.stringValue = NSLocalizedString(@"FileThumbnail", nil);
    self.thumbView.delegate = self;
    self.thumbView.dataSource = self;
    [self.thumbView setSelectable:YES];
    [self.thumbView registerClass:[ThumbItem class] forItemWithIdentifier:@"ThumbItem"];
}

- (void)loadThumb:(NSString *)fid {
    self.thumbUrls = [[BoardManager sharedInstance].boardController getThumbnailImages:fid];
    if(self.thumbUrls.count != 0){
        self.thumbTip.stringValue = @"";
    }
    else {
        self.thumbTip.stringValue = NSLocalizedString(@"FileNotSupportThumbnail", nil);
    }
    __weak typeof(self) ws = self;
    // 快速连续两次更新会导致列表更新失败
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(1 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        [ws.thumbView reloadData];
    });
}

- (NSInteger)collectionView:(nonnull NSCollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    return self.thumbUrls.count;
}

- (nonnull NSCollectionViewItem *)collectionView:(nonnull NSCollectionView *)collectionView itemForRepresentedObjectAtIndexPath:(nonnull NSIndexPath *)indexPath {
    ThumbItem *item = [collectionView makeItemWithIdentifier:@"ThumbItem" forIndexPath:indexPath];
    NSString *url = self.thumbUrls[indexPath.item];
    dispatch_async(dispatch_get_global_queue(0, 0), ^{
        NSData *data = [NSData dataWithContentsOfURL:[NSURL URLWithString:url]];
        if(data && data.length > 0) {
            NSImage *img = [[NSImage alloc] initWithData:data];
            if(img) {
                dispatch_async(dispatch_get_main_queue(), ^{
                    [item.imageView setImage:img];
                });
            }
        }
    });
    return item;
}

- (void)collectionView:(NSCollectionView *)collectionView didSelectItemsAtIndexPaths:(NSSet<NSIndexPath *> *)indexPaths{
    TEduBoardController *controller = [BoardManager sharedInstance].boardController;
    NSString *fileId = [controller getCurrentFile];
    NSArray *boarIds = [controller getFileBoardList:fileId];
    NSString *boardId = boarIds[indexPaths.allObjects[0].item];
    [controller gotoBoard:boardId];
}
@end
